@@ -50,7 +50,6 @@ struct _PersonalPhotoFramePrivate
 {
     GtkWidget *homeWidget;
 	GtkWidget *event;
-    GtkWidget *contents;
 	
 	gboolean singleMode;
 	gchar *singleFilename;
@@ -58,11 +57,16 @@ struct _PersonalPhotoFramePrivate
 	gint slideshowNum;
 	gint slideshowDuration;
 	gboolean slideshowRandom;
+	gboolean showSlideshowControls;
 	gboolean isPressed;
 	
 	GtkWidget *singleModeBox;
 	GtkWidget *slideshowModeBox;
 	
+	GdkPixbuf *image;
+	GdkPixbuf *forwardButton;
+	GdkPixbuf *backButton;
+	
 	GtkWidget *settingsDialog;
 	
 	gboolean isShown;
@@ -74,6 +78,35 @@ HD_DEFINE_PLUGIN_MODULE (PersonalPhotoFrame, personal_photo_frame, HD_TYPE_HOME_
 gboolean personal_photo_frame_timeout (PersonalPhotoFrame *self);
 void personal_photo_frame_update_content (PersonalPhotoFrame *self);
 
+void personal_photo_frame_remove_timers (PersonalPhotoFrame *self)
+{
+	//g_warning ("%s", __PRETTY_FUNCTION__);
+	if (self->priv->eventID != 0) {
+		g_source_remove (self->priv->eventID);
+		self->priv->eventID = 0;
+	}
+}
+
+void personal_photo_frame_add_timers (PersonalPhotoFrame *self)
+{
+	//g_warning ("%s", __PRETTY_FUNCTION__);
+	if (self->priv->eventID == 0) {
+		if (self->priv->slideshowDuration == 1) {
+			self->priv->eventID = g_timeout_add_seconds (30, (GSourceFunc)personal_photo_frame_timeout, self);
+		} else if (self->priv->slideshowDuration == 2) {
+			self->priv->eventID = g_timeout_add_seconds (60, (GSourceFunc)personal_photo_frame_timeout, self);
+		} else if (self->priv->slideshowDuration == 3) {
+			self->priv->eventID = g_timeout_add_seconds (300, (GSourceFunc)personal_photo_frame_timeout, self);
+		} else if (self->priv->slideshowDuration == 4) {
+			self->priv->eventID = g_timeout_add_seconds (600, (GSourceFunc)personal_photo_frame_timeout, self);
+		} else if (self->priv->slideshowDuration == 5) {
+			self->priv->eventID = g_timeout_add_seconds (1800, (GSourceFunc)personal_photo_frame_timeout, self);
+		} else if (self->priv->slideshowDuration == 6) {
+			self->priv->eventID = g_timeout_add_seconds (3600, (GSourceFunc)personal_photo_frame_timeout, self);
+		}
+	}
+}
+
 void personal_photo_frame_read_settings ( PersonalPhotoFrame *self )
 {
 	//g_warning("%s", __PRETTY_FUNCTION__);
@@ -106,6 +139,12 @@ void personal_photo_frame_read_settings ( PersonalPhotoFrame *self )
 			g_error_free (error);
 			error = NULL;
 		}
+		self->priv->slideshowNum = g_key_file_get_integer (keyFile, "config", "slideshowNum", &error);
+		if (error) {
+			self->priv->slideshowNum = 0;
+			g_error_free (error);
+			error = NULL;
+		}
 		self->priv->slideshowDuration = g_key_file_get_integer (keyFile, "config", "slideshowDuration", &error);
 		if (error) {
 			self->priv->slideshowDuration = 0;
@@ -118,30 +157,22 @@ void personal_photo_frame_read_settings ( PersonalPhotoFrame *self )
 			g_error_free (error);
 			error = NULL;
 		}
+		self->priv->showSlideshowControls = g_key_file_get_boolean (keyFile, "config", "showSlideshowControls", &error);
+		if (error) {
+			self->priv->showSlideshowControls = TRUE;
+			g_error_free (error);
+			error = NULL;
+		}
     } else {
 		self->priv->singleMode = TRUE;
         self->priv->singleFilename = "/usr/share/icons/hicolor/scalable/hildon/personal-photo-frame.png";
 		self->priv->slideshowDirname = "";
+		self->priv->slideshowNum = 0;
 		self->priv->slideshowDuration = 0;
 		self->priv->slideshowRandom = FALSE;
+		self->priv->showSlideshowControls = TRUE;
     }
 	
-	if (!self->priv->singleMode) {
-		if (self->priv->slideshowDuration == 1) {
-			self->priv->eventID = g_timeout_add (30 * 1000, (GSourceFunc)personal_photo_frame_timeout, self);
-		} else if (self->priv->slideshowDuration == 2) {
-			self->priv->eventID = g_timeout_add (60 * 1000, (GSourceFunc)personal_photo_frame_timeout, self);
-		} else if (self->priv->slideshowDuration == 3) {
-			self->priv->eventID = g_timeout_add (300 * 1000, (GSourceFunc)personal_photo_frame_timeout, self);
-		} else if (self->priv->slideshowDuration == 4) {
-			self->priv->eventID = g_timeout_add (600 * 1000, (GSourceFunc)personal_photo_frame_timeout, self);
-		} else if (self->priv->slideshowDuration == 5) {
-			self->priv->eventID = g_timeout_add (1800 * 1000, (GSourceFunc)personal_photo_frame_timeout, self);
-		} else if (self->priv->slideshowDuration == 6) {
-			self->priv->eventID = g_timeout_add (3600 * 1000, (GSourceFunc)personal_photo_frame_timeout, self);
-		}
-	}
-
     g_key_file_free (keyFile);
     g_free (filename);
 }
@@ -160,8 +191,10 @@ void personal_photo_frame_write_settings (PersonalPhotoFrame *self)
 	g_key_file_set_boolean (keyFile, "config", "singleMode", self->priv->singleMode);
 	g_key_file_set_string (keyFile, "config", "singleFilename", self->priv->singleFilename);
 	g_key_file_set_string (keyFile, "config", "slideshowDirname", self->priv->slideshowDirname);
+	g_key_file_set_integer (keyFile, "config", "slideshowNum", self->priv->slideshowNum);
 	g_key_file_set_integer (keyFile, "config", "slideshowDuration", self->priv->slideshowDuration);
 	g_key_file_set_boolean (keyFile, "config", "slideshowRandom", self->priv->slideshowRandom);
+	g_key_file_set_boolean (keyFile, "config", "showSlideshowControls", self->priv->showSlideshowControls);
 	
 	filename = g_strconcat (HOME_DIR, "/.personal_photo_frame", NULL);
 	fileData = g_key_file_to_data (keyFile, &size, NULL);
@@ -184,6 +217,27 @@ void personal_photo_frame_button_release (GtkWidget *widget, GdkEventButton *eve
 {
 	//g_warning ("%s", __PRETTY_FUNCTION__);
 	self->priv->isPressed = FALSE;
+	
+	if (!self->priv->singleMode && self->priv->showSlideshowControls) {
+		gdouble x_coord, y_coord;
+	
+		if (gdk_event_get_coords (event, &x_coord, &y_coord)) {
+			if (y_coord > 310.0 - 48.0) {
+				if (x_coord > 310.0 - 48.0) {
+					self->priv->slideshowNum++;
+					personal_photo_frame_update_content (self);
+					personal_photo_frame_remove_timers (self);
+					personal_photo_frame_add_timers (self);
+				} else if (x_coord < 5.0 + 48.0) {
+					self->priv->slideshowNum-=2;
+					personal_photo_frame_update_content (self);
+					personal_photo_frame_remove_timers (self);
+					personal_photo_frame_add_timers (self);
+				}
+			}
+		}
+	}
+	
 	gtk_widget_queue_draw (GTK_WIDGET (self));
 }
 
@@ -201,12 +255,13 @@ void personal_photo_frame_content_create (PersonalPhotoFrame *self)
 	gtk_event_box_set_visible_window (GTK_EVENT_BOX (self->priv->event), FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (self->priv->event), 5);
 
-	self->priv->contents = gtk_image_new ();
-	gtk_misc_set_alignment (GTK_MISC (self->priv->contents), 0.5, 0.5);
-
-	gtk_container_add (GTK_CONTAINER (self->priv->event), self->priv->contents);
 	gtk_box_pack_start (GTK_BOX (self->priv->homeWidget), self->priv->event, TRUE, TRUE, 0);
 	
+	self->priv->image = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, 305, 305);
+	gdk_pixbuf_fill (self->priv->image, 0x00000000);
+	self->priv->backButton = gdk_pixbuf_new_from_file ("/usr/share/icons/hicolor/48x48/hildon/general_back.png", NULL);
+	self->priv->forwardButton = gdk_pixbuf_new_from_file ("/usr/share/icons/hicolor/48x48/hildon/general_forward.png", NULL);
+	
 	g_signal_connect (self->priv->event, "button-release-event", G_CALLBACK (personal_photo_frame_button_release), self);
 	g_signal_connect (self->priv->event, "button-press-event", G_CALLBACK (personal_photo_frame_button_press), self);
 	g_signal_connect (self->priv->event, "leave-notify-event", G_CALLBACK (personal_photo_frame_leave_event), self);
@@ -218,12 +273,9 @@ gboolean personal_photo_frame_timeout (PersonalPhotoFrame *self)
 {
 	//g_warning ("%s", __PRETTY_FUNCTION__);
 	personal_photo_frame_update_content (self);
-	self->priv->slideshowNum++;
 	
 	if (!self->priv->isShown) {
-		if (self->priv->eventID != 0) {
-			self->priv->eventID = 0;
-		}
+		personal_photo_frame_remove_timers (self);
 	}
 	return self->priv->isShown;
 }
@@ -233,45 +285,48 @@ void personal_photo_frame_update_content (PersonalPhotoFrame *self)
 	//g_warning("%s", __PRETTY_FUNCTION__);
 
 	if (self->priv->singleMode) {
-		GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file_at_scale (self->priv->singleFilename, 305, 305, TRUE, NULL);
-		gtk_image_set_from_pixbuf (GTK_IMAGE (self->priv->contents), pixbuf);
-		g_object_unref (pixbuf);
+		g_object_unref (self->priv->image);
+		self->priv->image = gdk_pixbuf_new_from_file_at_scale (self->priv->singleFilename, 305, 305, TRUE, NULL);
 	} else {
 		GDir *dir = g_dir_open (self->priv->slideshowDirname, 0, NULL);
 		
 		if (dir) {
 			gboolean found = FALSE;
-			gboolean looped = FALSE;
-			gint start = self->priv->slideshowNum;
+			gint looped = 0;
 			
-			while ((!found) && (!looped)) {
+			while ((!found) && (looped < 2)) {
 				int i;
 				for (i=0; i<self->priv->slideshowNum; i++) {
 					g_dir_read_name (dir);
 				}
+				self->priv->slideshowNum++;
 				gchar *filename = g_strdup (g_dir_read_name (dir));
 				if (!filename) {
-					self->priv->slideshowNum = -1;
+					self->priv->slideshowNum = 0;
+					looped++;
 					g_dir_rewind (dir);
-					filename = g_strdup (g_dir_read_name (dir));
-				}
-				gchar *fullName = g_strconcat (self->priv->slideshowDirname, "/", filename, NULL);
-
-				GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file_at_scale (fullName, 305, 305, TRUE, NULL);
-				if (pixbuf) {
-					gtk_image_set_from_pixbuf (GTK_IMAGE (self->priv->contents), pixbuf);
-					g_object_unref (pixbuf);
-					found = TRUE;
+					g_object_unref (self->priv->image);
+					self->priv->image = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, 305, 305);
+					gdk_pixbuf_fill (self->priv->image, 0x00000000);
 				} else {
-					self->priv->slideshowNum++;
-					if (self->priv->slideshowNum == start) {
-						looped = TRUE;
+					gchar *fullName = g_strconcat (self->priv->slideshowDirname, "/", filename, NULL);
+
+					GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file_at_scale (fullName, 305, 305, TRUE, NULL);
+					if (pixbuf) {
+						g_object_unref (self->priv->image);
+						self->priv->image = gdk_pixbuf_copy (pixbuf);
+						g_object_unref (pixbuf);
+						found = TRUE;
+						personal_photo_frame_write_settings (self);
 					}
+					g_free (filename);
+					g_free (fullName);
 				}
-				g_free (filename);
-				g_free (fullName);
 			}
 			g_dir_close (dir);
+		} else {
+			g_object_unref (self->priv->image);
+			self->priv->image = gdk_pixbuf_new_from_file_at_scale ("/usr/share/icons/hicolor/scalable/hildon/personal-photo-frame.png", 305, 305, TRUE, NULL);
 		}
 	}
 }
@@ -367,9 +422,12 @@ void personal_photo_frame_settings (HDHomePluginItem *hitem, PersonalPhotoFrame
 	hildon_check_button_set_active (HILDON_CHECK_BUTTON (slideshowRandom), self->priv->slideshowRandom);
 	//gtk_box_pack_start (GTK_BOX (self->priv->slideshowModeBox), slideshowRandom, FALSE, FALSE, 0);
 	
+	GtkWidget *showSlideshowControls = hildon_check_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
+	gtk_button_set_label (GTK_BUTTON (showSlideshowControls), "Display slide show controls");
+	hildon_check_button_set_active (HILDON_CHECK_BUTTON (showSlideshowControls), self->priv->showSlideshowControls);
+	gtk_box_pack_start (GTK_BOX (self->priv->slideshowModeBox), showSlideshowControls, FALSE, FALSE, 0);
 	
 	gtk_box_pack_start (GTK_BOX (content_area), hbox, FALSE, FALSE, 0);
-	//gtk_box_pack_start (GTK_BOX (content_area), self->priv->singleModeBox, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (content_area), self->priv->slideshowModeBox, FALSE, FALSE, 0);
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (singleModeToggle), self->priv->singleMode);
@@ -381,36 +439,24 @@ void personal_photo_frame_settings (HDHomePluginItem *hitem, PersonalPhotoFrame
 	personal_photo_frame_mode_toggle (singleModeToggle, self);
 	
 	if (gtk_dialog_run (GTK_DIALOG (self->priv->settingsDialog)) == GTK_RESPONSE_ACCEPT) {
+		gchar *compare = g_strdup (self->priv->slideshowDirname);
 		self->priv->singleMode = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (singleModeToggle));
 		self->priv->singleFilename = g_strdup (hildon_button_get_value (HILDON_BUTTON (singlePhotoButton)));
 		self->priv->slideshowDirname = g_strdup (hildon_button_get_value (HILDON_BUTTON (slideshowDirButton)));
 		self->priv->slideshowDuration = hildon_touch_selector_get_active (HILDON_TOUCH_SELECTOR (selector), 0);
 		self->priv->slideshowRandom = hildon_check_button_get_active (HILDON_CHECK_BUTTON (slideshowRandom));
-		self->priv->slideshowNum = 0;
+		self->priv->showSlideshowControls = hildon_check_button_get_active (HILDON_CHECK_BUTTON (showSlideshowControls));
+		
+		if (!g_str_equal (compare, self->priv->slideshowDirname)) {
+			self->priv->slideshowNum = 0;
+		}
+		g_free (compare);
 		
 		personal_photo_frame_write_settings (self);
 		personal_photo_frame_update_content (self);
 		
-		if (self->priv->eventID != 0) {
-			g_source_remove (self->priv->eventID);
-			self->priv->eventID = 0;
-		}
-		
-		if (!self->priv->singleMode) {
-			if (self->priv->slideshowDuration == 1) {
-				self->priv->eventID = g_timeout_add (30 * 1000, (GSourceFunc)personal_photo_frame_timeout, self);
-			} else if (self->priv->slideshowDuration == 2) {
-				self->priv->eventID = g_timeout_add (60 * 1000, (GSourceFunc)personal_photo_frame_timeout, self);
-			} else if (self->priv->slideshowDuration == 3) {
-				self->priv->eventID = g_timeout_add (300 * 1000, (GSourceFunc)personal_photo_frame_timeout, self);
-			} else if (self->priv->slideshowDuration == 4) {
-				self->priv->eventID = g_timeout_add (600 * 1000, (GSourceFunc)personal_photo_frame_timeout, self);
-			} else if (self->priv->slideshowDuration == 5) {
-				self->priv->eventID = g_timeout_add (1800 * 1000, (GSourceFunc)personal_photo_frame_timeout, self);
-			} else if (self->priv->slideshowDuration == 6) {
-				self->priv->eventID = g_timeout_add (3600 * 1000, (GSourceFunc)personal_photo_frame_timeout, self);
-			}
-		}
+		personal_photo_frame_remove_timers (self);
+		personal_photo_frame_add_timers (self);
 	}
 	gtk_widget_destroy (self->priv->settingsDialog);
 }
@@ -426,20 +472,8 @@ static void personal_photo_frame_check_desktop (GObject *gobject, GParamSpec *ps
 			if (!self->priv->singleMode) {
 				if (self->priv->slideshowDuration == 0) {
 					personal_photo_frame_update_content (self);
-					self->priv->slideshowNum++;
-				} else if (self->priv->slideshowDuration == 1) {
-					self->priv->eventID = g_timeout_add (30 * 1000, (GSourceFunc)personal_photo_frame_timeout, self);
-				} else if (self->priv->slideshowDuration == 2) {
-					self->priv->eventID = g_timeout_add (60 * 1000, (GSourceFunc)personal_photo_frame_timeout, self);
-				} else if (self->priv->slideshowDuration == 3) {
-					self->priv->eventID = g_timeout_add (300 * 1000, (GSourceFunc)personal_photo_frame_timeout, self);
-				} else if (self->priv->slideshowDuration == 4) {
-					self->priv->eventID = g_timeout_add (600 * 1000, (GSourceFunc)personal_photo_frame_timeout, self);
-				} else if (self->priv->slideshowDuration == 5) {
-					self->priv->eventID = g_timeout_add (1800 * 1000, (GSourceFunc)personal_photo_frame_timeout, self);
-				} else if (self->priv->slideshowDuration == 6) {
-					self->priv->eventID = g_timeout_add (3600 * 1000, (GSourceFunc)personal_photo_frame_timeout, self);
 				}
+				personal_photo_frame_add_timers (self);
 			}
 		}
 	}
@@ -461,6 +495,10 @@ static void personal_photo_frame_finalize (GObject *object)
 	if (self->priv->eventID) {
 		g_source_remove (self->priv->eventID);
 	}
+	g_object_unref (self->priv->forwardButton);
+	g_object_unref (self->priv->backButton);
+	
+	personal_photo_frame_write_settings (self);
 
 	G_OBJECT_CLASS (personal_photo_frame_parent_class)->finalize (object);
 }
@@ -524,8 +562,24 @@ static gboolean personal_photo_frame_expose_event (GtkWidget *widget, GdkEventEx
 	cairo_set_line_width (cr, 1);
 	cairo_stroke (cr);
 	
+	gint pixbufWidth=gdk_pixbuf_get_width(self->priv->image);
+	gint pixbufHeight=gdk_pixbuf_get_height(self->priv->image);
+	gdk_cairo_set_source_pixbuf (cr, self->priv->image, 5 + ((305-pixbufWidth)/2), 5 + ((305-pixbufHeight)/2));
+	cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+	cairo_paint (cr);
+	
 	cairo_destroy(cr);
-
+	
+	if (!self->priv->singleMode && self->priv->showSlideshowControls) {
+		cr = gdk_cairo_create(GDK_DRAWABLE (widget->window));
+		gdk_cairo_set_source_pixbuf (cr, self->priv->backButton, 5, 310 - 48);
+		cairo_paint_with_alpha (cr, 0.5);
+		
+		gdk_cairo_set_source_pixbuf (cr, self->priv->forwardButton, 310 -48, 310 - 48);
+		cairo_paint_with_alpha (cr, 0.5);
+		cairo_destroy (cr);
+	}
+	
 	return GTK_WIDGET_CLASS (personal_photo_frame_parent_class)->expose_event (widget, event);
 }
 
@@ -554,7 +608,6 @@ static void personal_photo_frame_init (PersonalPhotoFrame *self)
 	self->priv = PERSONAL_PHOTO_FRAME_GET_PRIVATE (self);
 	self->priv->eventID = 0;
 	self->priv->isPressed = FALSE;
-	self->priv->slideshowNum = 0;
 	
 	hd_home_plugin_item_set_settings (&self->parent, TRUE);
 	g_signal_connect (&self->parent, "show-settings", G_CALLBACK (personal_photo_frame_settings), self);
@@ -574,6 +627,7 @@ static void personal_photo_frame_init (PersonalPhotoFrame *self)
 	
 	personal_photo_frame_read_settings (self);
 	personal_photo_frame_content_create (self);
+	personal_photo_frame_add_timers (self);
 	personal_photo_frame_update_content (self);
 	
 	g_signal_connect (self, "notify::is-on-current-desktop", G_CALLBACK (personal_photo_frame_check_desktop), self);
